//===========================================================================
// Simulationsbeispiel
//===========================================================================

//===========================================================================
// includes
//===========================================================================

// Deklaration einbinden
#include "particle.h"


//===========================================================================
// implementierung
//===========================================================================

CParticle::CParticle()
{
    // Initialisierung der Vektoren findet automatisch
    // durch deren Konstruktor statt
}

CParticle::~CParticle()
{
    // nichts aufzurumen
}


void CParticle::moveTo(CVertex position)
{
    // Auf Position setzen, Geschwindigkeit und Beschleunigung lschen
    m_position     = position;
    m_velocity     = CVertex(0,0,0);
    m_acceleration = CVertex(0,0,0);
}


void CParticle::addForce(CVertex force)
{
    // Windkraft auf Beschleunigung addieren (gedachte Masse = 1)
    m_acceleration += force;
}


void CParticle::accelerateTowards(CVertex destination, Coord accelerationFactor)
{
    // Auf Punkt zu beschleunigen (gedachte Masse = 1/accelerationFactor)
    // Vorsicht, da der Richtungsvektor normalisiert wird,
    // schiet der Vektor ber das Ziel hinaus!
    m_acceleration += (destination - m_position).normalize() * accelerationFactor;
}


void CParticle::accelerateProportional(CVertex destination, Coord accelerationFactor)
{
    // Auf Punkt zu beschleunigen (gedachte Masse = 1/accelerationFactor)
    m_acceleration += (destination - m_position) * accelerationFactor;
}


void CParticle::update(Coord timeStep, Coord airFriction)
{
    // v = v + a*dt
    m_velocity     += m_acceleration  * timeStep;

    // x = x + v*dt
    m_position     += m_velocity * timeStep;

    // Luftwiderstand bercksichtigen (warum erst jetzt?)
    m_velocity     -= m_velocity * (timeStep * airFriction);

    // Beschleunigung auf 0 zurcksetzen fr nchsten Zeitschritt
    m_acceleration  = CVertex(0,0,0);
}


CVertex CParticle::position() const
{
    // Aktuelle Position zurckgeben
    return m_position;
}
