//===========================================================================
// Vektorenbeispiel
//===========================================================================

// mehrfaches Einbinden durch Prprozessor verhindern
#ifndef _VERTEX_H_
#define _VERTEX_H_

// Mathematikbibliothek einbinden
#include <math.h>

// Typen for Vektorkoordinaten definieren
typedef float Coord;

// deklariere eine Klasse fr Vektoren
class CVertex
{
  public:

    CVertex(Coord initX=0.0, Coord initY=0.0, Coord initZ=0.0)
    { x = initX; y = initY; z = initZ; }

    virtual ~CVertex() { }

    // Grundrechenarten berladen
    CVertex operator+(const CVertex& v) const { return CVertex(x+v.x, y+v.y, z+v.z); }
    CVertex operator-(const CVertex& v) const { return CVertex(x-v.x, y-v.y, z-v.z); }
    CVertex operator*(const Coord s) const { return CVertex(x*s, y*s, z*s); }
    CVertex operator/(const Coord s) const { return CVertex(x/s, y/s, z/s); }

    // Zuweisungen berladen
    const CVertex& operator+=(const CVertex& v) { x += v.x; y += v.y; z += v.z; return *this; }
    const CVertex& operator-=(const CVertex& v) { x -= v.x; y -= v.y; z -= v.z; return *this; }
    const CVertex& operator*=(const Coord s) { x *= s; y *= s; z *= s; return *this; }
    const CVertex& operator/=(const Coord s) { x /= s; y /= s; z /= s; return *this; }

    // Vergleiche berladen
    bool operator==(const CVertex& v) const { return (x == v.x) && (y == v.y) && (z == v.z); }
    bool operator!=(const CVertex& v) const { return !operator==(v); }

    // Unre Oeratoren berladen
    CVertex operator-() const { return CVertex(-x, -y, -z); }

    // Skalarprodukt
    Coord operator*(const CVertex& v) const { return x*v.x + y*v.y + z*v.z; }

    // Kreuzprodukt
    CVertex cross(const CVertex& v) const
    {
        return CVertex(y*v.z - z*v.y,
                       z*v.x - x*v.z,
                       x*v.y - y*v.x);
    }

    // Lnge berechnen
    Coord length() const { return sqrt(x*x + y*y + z*z); }

    // Vektor normalisieren
    CVertex normalize() const
    {
        Coord l = length();
        if (l != 0.0f) return *this/l;
        else           return *this;
    }

    // Die drei Koordinaten unseres Vektors
    Coord x, y, z;

}; // Klassendeklaration auch mit Semikolon abschlieen

#endif // _VERTEX_H_
